<?php

namespace Justlab\JLToken\Controllers;

use Justlab\JLToken\DB;
use Justlab\JLToken\Models\Call;
use Justlab\JLToken\Models\Counter;
use Justlab\JLToken\Models\Service;
use Justlab\JLToken\Repositories\CallRepository;
use Justlab\JLToken\Repositories\CounterRepository;
use Justlab\JLToken\Repositories\ServiceRepository;
use Justlab\JLToken\Repositories\TokenRepository;
use Justlab\JLToken\Validator\Validator;
use Throwable;

if (!function_exists('add_action') || !defined('ABSPATH')) {
    echo "Hi there! I'm just a plugin, not much I can do when called directly.";
    exit;
}

class CallController extends Controller
{
    const VIEW_DIR = JLTOKEN__PLUGIN_DIR . 'views/admin/call/';

    private static $initiated = false;

    public static function init()
    {
        if (!self::$initiated) {
            self::initHooks();
        }
    }

    public static function initHooks()
    {
        self::$initiated = true;

        add_submenu_page('jltoken', esc_html__('Call', 'jltoken') . ' | JL Token', esc_html__('Call', 'jltoken'), 'manage_options', 'jltoken-call', [self::class, 'view'], 1);

        if (isset($_GET['page']) && ($_GET['page'] == 'jltoken-call')) {
            self::enqueueAssets();
        }
    }

    public static function view()
    {
        self::call();

        $services = ServiceRepository::getActiveServices();
        $counters = CounterRepository::getActiveCounters();

        ob_start();

        include_once(self::VIEW_DIR . 'call.php');

        $view = ob_get_contents();

        ob_end_clean();

        echo $view;
    }

    public static function setServiceAndCounter()
    {
        self::call();

        check_ajax_referer('jltoken_set_service_counter');

        $data = [];
        $data['service_id'] = isset($_POST['service_id']) ? trim($_POST['service_id']) : '';
        $data['counter_id'] = isset($_POST['counter_id']) ? trim($_POST['counter_id']) : '';

        $validator = new Validator;
        $validation = $validator->validate($data, [
            'service_id' => 'required|exists:services,id',
            'counter_id' => 'required|exists:counters,id',
        ]);

        if ($validation->fails()) {
            wp_send_json($validation->errors(), 422);
        }

        $service = Service::find($data['service_id']);
        $counter = Counter::find($data['counter_id']);

        $_SESSION['jltoken_service'] = $service;
        $_SESSION['jltoken_counter'] = $counter;

        wp_send_json(['service' => $service, 'counter' => $counter, 'tokens_for_call' => TokenRepository::getTokensForCall($service), 'called_tokens' => TokenRepository::getCalledTokens($service, $counter)]);
    }

    public static function getTokensForCall()
    {
        self::call();

        check_ajax_referer('jltoken_get_token_for_call');

        $service = $_SESSION['jltoken_service'];
        $counter = $_SESSION['jltoken_counter'];

        $tokens_for_call = [];
        $called_tokens = [];
        if ($service && $counter) {
            $tokens_for_call = TokenRepository::getTokensForCall($service);
            $called_tokens = TokenRepository::getCalledTokens($service, $counter);
        }

        wp_send_json(['service' => $service, 'counter' => $counter, 'tokens_for_call' => $tokens_for_call, 'called_tokens' => $called_tokens]);
    }

    public static function callNext()
    {
        self::call();

        check_ajax_referer('jltoken_call_next');

        $data = [];
        $data['service_id'] = isset($_POST['service_id']) ? trim($_POST['service_id']) : '';
        $data['counter_id'] = isset($_POST['counter_id']) ? trim($_POST['counter_id']) : '';
        $data['by_id'] = isset($_POST['by_id']) ? trim($_POST['by_id']) : '';
        $data['queue_id'] = isset($_POST['queue_id']) ? trim($_POST['queue_id']) : '';

        $validator = new Validator;
        $validation = $validator->validate($data, [
            'service_id' => 'required|exists:services,id',
            'counter_id' => 'required_if:by_id,false|exists:counters,id',
            'by_id' => 'required|boolean',
            'queue_id' => 'required_if:by_id,true|exists:queues,id',
        ]);

        if ($validation->fails()) {
            wp_send_json($validation->errors(), 422);
        }

        $data['by_id'] = filter_var($data['by_id'], FILTER_VALIDATE_BOOLEAN);

        DB::beginTransaction();
        try {
            if ($data['by_id']) {
                $called = CallRepository::callnextTokenById($data['queue_id'], $data['service_id']);
            } else {
                $called = CallRepository::callNext($data['service_id'], $data['counter_id']);
            }

            if (!$called) {
                DB::rollback();

                wp_send_json(['no_token_found' => true]);
            }

            CallRepository::setCallsForDisplay($called->service_id);
            TokenRepository::setTokensOnFile();
        } catch (Throwable $th) {
            DB::rollback();

            wp_send_json(['status' => false], 500);
        }
        DB::commit();

        wp_send_json($called);
    }

    public static function serveToken()
    {
        self::call();

        check_ajax_referer('jltoken_serve_token');

        $data = [];
        $data['call_id'] = isset($_POST['call_id']) ? trim($_POST['call_id']) : '';

        $validator = new Validator;
        $validation = $validator->validate($data, [
            'call_id' => 'required|exists:calls,id',
        ]);

        if ($validation->fails()) {
            wp_send_json($validation->errors(), 422);
        }

        DB::beginTransaction();
        try {
            $call = Call::builder()
                ->where(['id' => $data['call_id']])
                ->where(['call_status_id' => NULL])
                ->first(ARRAY_A);
            $call = $call ? new Call($call) : $call;

            if (!$call) {
                DB::rollback();

                wp_send_json(['already_executed' => true]);
            }

            $call = CallRepository::serveToken($call);

            CallRepository::setCallsForDisplay($call->service_id);
            TokenRepository::setTokensOnFile();
        } catch (Throwable $th) {
            DB::rollback();

            wp_send_json(['status' => false], 500);
        }
        DB::commit();

        wp_send_json($call);
    }

    public static function noShowToken()
    {
        self::call();

        check_ajax_referer('jltoken_no_show');

        $data = [];
        $data['call_id'] = isset($_POST['call_id']) ? trim($_POST['call_id']) : '';

        $validator = new Validator;
        $validation = $validator->validate($data, [
            'call_id' => 'required|exists:calls,id',
        ]);

        if ($validation->fails()) {
            wp_send_json($validation->errors(), 422);
        }

        DB::beginTransaction();
        try {
            $call = Call::builder()
                ->where(['id' => $data['call_id']])
                ->where(['call_status_id' => NULL])
                ->first(ARRAY_A);
            $call = $call ? new Call($call) : $call;

            if (!$call) {
                DB::rollback();

                wp_send_json(['already_executed' => true]);
            }

            $call = CallRepository::noShowToken($call);

            CallRepository::setCallsForDisplay($call->service_id);
            TokenRepository::setTokensOnFile();
        } catch (Throwable $th) {
            DB::rollback();

            wp_send_json(['status' => false], 500);
        }
        DB::commit();

        wp_send_json($call);
    }

    public static function recallToken()
    {
        self::call();

        check_ajax_referer('jltoken_recall_token');

        $data = [];
        $data['call_id'] = isset($_POST['call_id']) ? trim($_POST['call_id']) : '';

        $validator = new Validator;
        $validation = $validator->validate($data, [
            'call_id' => 'required|exists:calls,id',
        ]);

        if ($validation->fails()) {
            wp_send_json($validation->errors(), 422);
        }

        DB::beginTransaction();
        try {
            $call = Call::find($data['call_id']);

            $call = CallRepository::recallToken($call);

            CallRepository::setCallsForDisplay($call->service_id);
            TokenRepository::setTokensOnFile();
        } catch (Throwable $th) {
            DB::rollback();

            wp_send_json(['status' => false], 500);
        }
        DB::commit();

        wp_send_json($call);
    }

    public static function enqueueAssets()
    {
        wp_enqueue_style('jltoken-materialize');
        wp_enqueue_style('jltoken-style');
        wp_enqueue_style('jltoken-main');
        wp_enqueue_style('jltoken-normalize');
        wp_enqueue_style('jltoken-app');

        wp_enqueue_script('jltoken-vendor');
        wp_enqueue_script('jltoken-app');
    }
}
